
// includes
#include "../io/Console.hpp"
#include "../platform/Window.hpp"

#include "OffscreenRenderTarget.hpp"

using namespace bleifrei::io;
using namespace bleifrei::platform;
using namespace bleifrei;

namespace bleifrei {
	namespace gfx {

		OffscreenRenderTarget::OffscreenRenderTarget(int width, int height, Format format)
		{
			this->width = width;
			this->height = height;

			unsigned int count = 0;
			int	pixelFormat;

			HDC windowDC = Window::getInstance().windows_getDeviceContextHandle();

			if (!WGL_ARB_pixel_format_initialized)
			{
				Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_ARB_pixel_format not avilable!");
				return;
			}
			if (!WGL_ARB_pbuffer_initialized)
			{
				Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_ARB_pbuffer not avilable!");
				return;
			}
			if (!WGL_ARB_render_texture_initialized)
			{
				Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_ARB_render_texture not avilable!");
				return;
			}

			int pAttrib[] = {
				WGL_TEXTURE_FORMAT_ARB,		WGL_TEXTURE_RGBA_ARB,
				WGL_TEXTURE_TARGET_ARB,		WGL_TEXTURE_2D_ARB,
				0
			};

			if (format == rgba8)
			{
				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,				TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,			TRUE,
					WGL_BIND_TO_TEXTURE_RGBA_ARB,		TRUE,
					WGL_RED_BITS_ARB,					8,
					WGL_GREEN_BITS_ARB,					8,
					WGL_BLUE_BITS_ARB,					8,
					WGL_ALPHA_BITS_ARB,					8,
					WGL_DEPTH_BITS_ARB,					24,
					WGL_DOUBLE_BUFFER_ARB,				FALSE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count))
				{
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}
			}
			else if (format == rgba8rect) {
				if (!WGL_NV_render_texture_rectangle_initialized)
				{
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_NV_render_texture_rectangle not avilable!");
					return;
				}

				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,				TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,			TRUE,
					WGL_BIND_TO_TEXTURE_RECTANGLE_RGBA_NV,	TRUE,
					WGL_RED_BITS_ARB,					8,
					WGL_GREEN_BITS_ARB,					8,
					WGL_BLUE_BITS_ARB,					8,
					WGL_ALPHA_BITS_ARB,					8,
					WGL_DEPTH_BITS_ARB,					24,
					WGL_DOUBLE_BUFFER_ARB,				FALSE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count)) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}

				pAttrib[3] = WGL_TEXTURE_RECTANGLE_NV;
			}
			else if (format == rgb8)
			{
				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,				TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,			TRUE,
					WGL_BIND_TO_TEXTURE_RGBA_ARB,		TRUE,
					WGL_RED_BITS_ARB,					8,
					WGL_GREEN_BITS_ARB,					8,
					WGL_BLUE_BITS_ARB,					8,
					WGL_DEPTH_BITS_ARB,					24,
					WGL_DOUBLE_BUFFER_ARB,				FALSE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count))
				{
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}

				pAttrib[1] = WGL_TEXTURE_RGB_ARB;
			}
			else if (format == rgba8db)
			{
				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,				TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,			TRUE,
					WGL_BIND_TO_TEXTURE_RGBA_ARB,		TRUE,
					WGL_RED_BITS_ARB,					8,
					WGL_GREEN_BITS_ARB,					8,
					WGL_BLUE_BITS_ARB,					8,
					WGL_ALPHA_BITS_ARB,					8,
					WGL_DEPTH_BITS_ARB,					24,
					WGL_DOUBLE_BUFFER_ARB,				TRUE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count)) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}
			}
			else if (format == rgba8dbrect) {
				if (!WGL_NV_render_texture_rectangle_initialized) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_NV_render_texture_rectangle not avilable!");
					return;
				}

				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,				TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,			TRUE,
					WGL_BIND_TO_TEXTURE_RECTANGLE_RGBA_NV,	TRUE,
					WGL_RED_BITS_ARB,					8,
					WGL_GREEN_BITS_ARB,					8,
					WGL_BLUE_BITS_ARB,					8,
					WGL_ALPHA_BITS_ARB,					8,
					WGL_DEPTH_BITS_ARB,					24,
					WGL_STENCIL_BITS_ARB,				8,
					WGL_DOUBLE_BUFFER_ARB,				TRUE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count)) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}

				pAttrib[3] = WGL_TEXTURE_RECTANGLE_NV;
			}
			else if (format == rgb16f) {
				if (!WGL_ATI_pixel_format_float_initialized) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_ATI_pixel_format_float not avilable!");
					return;
				}
				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,			TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,		TRUE,
					WGL_BIND_TO_TEXTURE_RGB_ARB,	TRUE,
					WGL_PIXEL_TYPE_ARB,				WGL_TYPE_RGBA_FLOAT_ATI,
					WGL_RED_BITS_ARB,				32,
					WGL_GREEN_BITS_ARB,				32,
					WGL_BLUE_BITS_ARB,				32,
					WGL_DEPTH_BITS_ARB,				24,
					WGL_DOUBLE_BUFFER_ARB,			FALSE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count)) {
					Console::getInstance().print(1.0f, 0.0f, 0.0f, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}

				pAttrib[1] = WGL_TEXTURE_RGB_ARB;
			}
			else if (format == rgb16frect) {
				if (!GL_NV_float_buffer_initialized) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): GL_NV_float_buffer not avilable!");
					return;
				}
				if (!WGL_NV_float_buffer_initialized) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_NV_float_buffer not avilable!");
					return;
				}
				if (!WGL_NV_render_texture_rectangle_initialized) {
					Console::getInstance().print(1, 0, 0, "Error(RenderTarget): WGL_NV_render_texture_rectangle not avilable!");
					return;
				}

				int attr[] = {
					WGL_SUPPORT_OPENGL_ARB,			TRUE,
					WGL_DRAW_TO_PBUFFER_ARB,		TRUE,
					WGL_BIND_TO_TEXTURE_RECTANGLE_FLOAT_RGB_NV, TRUE,
					WGL_FLOAT_COMPONENTS_NV,					TRUE,
					WGL_RED_BITS_ARB,				32,
					WGL_GREEN_BITS_ARB,				32,
					WGL_BLUE_BITS_ARB,				32,
					WGL_DEPTH_BITS_ARB,				24,
					WGL_DOUBLE_BUFFER_ARB,			FALSE,
					0
				};

				if (!wglChoosePixelFormatARB(windowDC, (const int*)attr, NULL, 1, &pixelFormat, &count)) {
					Console::getInstance().print(1.0f, 0.0f, 0.0f, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
					return;
				}

				pAttrib[1] = WGL_TEXTURE_FLOAT_RGB_NV;
				pAttrib[3] = WGL_TEXTURE_RECTANGLE_NV;
			}
			else {
				Console::getInstance().print(1, 0, 0, "Error(RenderTarget): Unknown format!");
			}

			if (count == 0) {
				Console::getInstance().print(1.0f, 0.0f, 0.0f, "Error(RenderTarget): Could not find suitable pbuffer pixel format!");
				return;
			}

			pb = wglCreatePbufferARB(windowDC, pixelFormat, width, height, pAttrib);

			if (!pb) {
				Console::getInstance().print(1.0f, 0.0f, 0.0f, "Error(RenderTarget): Could not create pbuffer handle!");
				return;
			}

			hDC = wglGetPbufferDCARB(pb);

			if (!hDC) {
				Console::getInstance().print(1.0f, 0.0f, 0.0f, "Error(RenderTarget): Could not create pbuffer device context!");
				return;
			}

			if (!WGL_ARB_make_current_read_initialized) {
				hRC = wglCreateContext(hDC);

				if (!hRC) {
					Console::getInstance().print(1.0f, 0.0f, 0.0f, "Error(RenderTarget): Could not create pbuffer RC!");
					return;
				}

/* this should be done outside?
				wglShareLists(windowDC, hRC);

				setActive(this);
					platform::forceResize(width, height);

					glEnable(GL_DEPTH_TEST);
					glDepthFunc(GL_LESS);
					glShadeModel(GL_SMOOTH);
					glEnable(GL_CULL_FACE);
				setActive(0);
*/
			}

			int w, h;
			wglQueryPbufferARB(pb, WGL_PBUFFER_WIDTH_ARB, &w);
			wglQueryPbufferARB(pb, WGL_PBUFFER_HEIGHT_ARB, &h);
			Console::getInstance().printf(1,1,0, "Info(RenderTarget): width = %i, height = %i", w, h);
		}

		OffscreenRenderTarget::~OffscreenRenderTarget(void)
		{
		    wglReleasePbufferDCARB(pb, hDC);
			wglDestroyPbufferARB(pb);
		}

	}
}

