
#ifndef _include_bleifrei_tex_Texture_hpp_
#define _include_bleifrei_tex_Texture_hpp_ 

#include "../Base.hpp"

namespace bleifrei {
	namespace img {
		class Image;
	}
	namespace tex {

		enum TextureFilter
		{
			TextureFilterNearest,
			TextureFilterBilinear,
			TextureFilterTrilinear,
			TextureFilterMax
		};

		const int MaxTextureUnits = 8;

		class Texture : public Base
		{
		public:
			void bind(unsigned int unit = 0);
			void forceBind(unsigned int unit = 0);

			static void reset(void);

			void setMinFilter(TextureFilter filter);
			void setMagFilter(TextureFilter filter);
			void setAnisotropicFiltering(float level);

			void loadFromFramebuffer(int x, int y, int width, int height);

			// opengl specific
			unsigned int opengl_getTextureID(void)					{ return m_id;						}
		protected:
			Texture(unsigned int type);

			void create(void);
			void destroy(void);

			void enable(void);
			void disable(void);
			static void setUnit(unsigned int unit);

			unsigned int m_id;
			unsigned int m_type;

			static unsigned int m_currentType;
			static int m_currentUnit;
			static Texture *m_textureUnit[MaxTextureUnits];
		};

		Texture *const nullTexture = 0;

		class Texture2 :
			public Texture
		{
		public:
			Texture2(void);
			~Texture2(void);

			void load(img::Image *image);
		};

		class TextureRect :
			public Texture
		{
		public:
			TextureRect(void);
			~TextureRect(void);
		};

		class TextureCube :
			public Texture
		{
		public:
			TextureCube(void);
			~TextureCube(void);

			void load(img::Image &neg_x, img::Image &pos_x,	img::Image &neg_y, img::Image &pos_y, img::Image &neg_z, img::Image &pos_z);
		};


	}
}



#endif //_include_bleifrei_tex_Texture_hpp_