
//-------------------------------------------------------------------------------------------------
// carscene-1.fx
//-------------------------------------------------------------------------------------------------

//-------------------------------------------------------------------------------------------------
// Parameters with semantics and default values.
//-------------------------------------------------------------------------------------------------
float4x4 model				: MODEL_MATRIX;
float4x4 viewProjection		: VIEW_PROJECTION_MATRIX;

float3 light0ambient		: LIGHT0_AMBIENT;
float3 light0diffuse		: LIGHT0_DIFFUSE;
float3 light0specular		: LIGHT0_SPECULAR;
float3 light0position		: LIGHT0_POSITION;

float3 materialAmbient		: MATERIAL_AMBIENT;
float3 materialDiffuse		: MATERIAL_DIFFUSE;
float3 materialSpecular		: MATERIAL_SPECULAR;
float materialShininess		: MATERIAL_SHININESS; 

float time					: TIME;
float3 cameraPosition		: CAMERA_POSITION;

sampler2D environmentMap	: ENVIRONMENT_MAP;


//-------------------------------------------------------------------------------------------------
// Input/output structures used by shaders. Each member has a standard semantic defined by the
// Cg shading language.
//-------------------------------------------------------------------------------------------------
struct VS_INPUT
{
	float4 position			: POSITION;
	float3 normal			: NORMAL;
	float2 tex0				: TEXCOORD0;
};

struct VS_OUTPUT
{
    float4 position			: POSITION;
    float2 tex0				: TEXCOORD0;
    float3 worldNormal		: TEXCOORD1;
    float3 worldPosition	: TEXCOORD2;
};

struct VS_SIMPLE_OUTPUT
{
    float4 position			: POSITION;
    float2 tex0				: TEXCOORD0;
};

//-------------------------------------------------------------------------------------------------
// samplerDiffuseBilin
//		Texture sampler for the diffuse texture map.
//-------------------------------------------------------------------------------------------------
sampler2D samplerDiffuseBilin : DIFFUSE_MAP = sampler_state
{
    MinFilter = Linear;
    MagFilter = Linear;
};

//-------------------------------------------------------------------------------------------------
// calculateAmbient
//		This Cg-function calculates the ambient lighting term of light 0.
//-------------------------------------------------------------------------------------------------
float3 calculateAmbient(float3 light0ambient, float3 materialAmbient)
{
	// $TASK 1A: Ambient Lighting
    return 1;
}

//-------------------------------------------------------------------------------------------------
// calculateDiffuse
//		This Cg-function calculates the diffuse lighting term of light 0.
//-------------------------------------------------------------------------------------------------
float3 calculateDiffuse(float3 light0diffuse, float3 materialDiffuse, float3 normal, float3 directionToLight)
{
	// $TASK 1B: Diffuse Lighting
	float intensity = 0;
    return 0 * intensity;
}

//-------------------------------------------------------------------------------------------------
// calculateSpecular
//		This Cg-function calculates the specular lighting term of light 0.
//-------------------------------------------------------------------------------------------------
float3 calculateSpecular(float3 light0specular, float3 materialSpecular, float materialShininess, float3 normal, const float3 directionFromLight, const float3 directionToEye)
{
	// calculate reflection vector
    float3 r = normalize(reflect(directionFromLight, normal));

	// $TASK 1C: Specular Lighting
	float intensity = 0;
	
    return 0 * intensity;
}

//-------------------------------------------------------------------------------------------------
// vsSimple
//		Vertex shader used for the shadowed plane in the scene. Transforms the vertices to screen
//		space and outputs unmodified texture coordinates.
//-------------------------------------------------------------------------------------------------
VS_SIMPLE_OUTPUT vsSimple(VS_INPUT indata, uniform float4x4 model, uniform float4x4 viewProjection)
{
	VS_SIMPLE_OUTPUT outdata = (VS_SIMPLE_OUTPUT)0;
	
	// transform the position
    float3 position = mul(model, indata.position).xyz;						// position (world space)
    outdata.position  = mul(viewProjection, float4(position, 1));		// position (projected)
    
	outdata.tex0 = indata.tex0;
	
	return outdata;
}

//-------------------------------------------------------------------------------------------------
// psSimple
//		Pixel shader used for the shadowed plane in the scene. Samples the diffuse map using
//		texture coordinate set 0.
//-------------------------------------------------------------------------------------------------
float4 psSimple(VS_SIMPLE_OUTPUT indata, uniform sampler2D samplerDiffuseBilin) : COLOR
{
    return tex2D(samplerDiffuseBilin, indata.tex0);
}

//-------------------------------------------------------------------------------------------------
// vsHemi
//		Vertex shader used for the car. Transforms the position and normal of the vertices and
//		calculates the lighting terms by evaluation the functions 'calculateAmbient', 
//		'calculateDiffuse' and 'calculateSpecular'.
//-------------------------------------------------------------------------------------------------
VS_OUTPUT vsLighting(VS_INPUT indata, uniform float3 cameraPosition, uniform float time, uniform float4x4 model, uniform float4x4 viewProjection, uniform float3 light0position, uniform float3 light0ambient, uniform float3 light0diffuse, uniform float3 light0specular, uniform float3 materialAmbient, uniform float3 materialDiffuse, uniform float3 materialSpecular, uniform float3 materialShininess)
{
	VS_OUTPUT outdata = (VS_OUTPUT)0;
	
	// transform the position
    float3 position = mul(model, indata.position).xyz;						// position (world space)
    outdata.worldPosition = position;
    outdata.position  = mul(viewProjection, float4(outdata.worldPosition, 1));		// position (projected)

	// transform the normal
    outdata.worldNormal = normalize(mul((float3x3)model, indata.normal));		// normal (world space)

    return outdata;
}

//-------------------------------------------------------------------------------------------------
// psHemi
//		Pixel shader used for the car. Outputs the lighting value calculated in the vertex shader
//		and interpolated across the triangle by the GPU.
//-------------------------------------------------------------------------------------------------
float4 psLighting(VS_OUTPUT indata, uniform float3 cameraPosition, uniform float3 light0position, uniform float3 light0ambient, uniform float3 light0diffuse, uniform float3 light0specular, uniform float3 materialAmbient, uniform float3 materialDiffuse, uniform float3 materialSpecular, uniform float3 materialShininess) : COLOR
{
    float4 outColor;
    
	// calculate vectors used for lighting
    float3 directionToEye = normalize(cameraPosition - indata.worldPosition);
    float3 directionToLight = normalize(light0position.xyz - indata.worldPosition);
    float3 normal = normalize(indata.worldNormal);

    // calculate the lighting terms
    float3 diffuse = calculateAmbient(light0ambient, materialAmbient)
				   + calculateDiffuse(light0diffuse, materialDiffuse, normal, directionToLight); 
				   
    float3 specular = calculateSpecular(light0specular, materialSpecular, materialShininess, normal, -directionToLight, directionToEye);
    
    outColor = float4(diffuse + specular, 1.0);

    return outColor;
}

//-------------------------------------------------------------------------------------------------
// simple
//		Technique used for the shadowed plane in the scene.
//-------------------------------------------------------------------------------------------------
technique simple
{
	pass p0
	{
		BlendEnable = false;
		CullFaceEnable = true;
		CullFace = Back;
		DepthMask = true;
		DepthTestEnable = true;

		VertexProgram = compile arbvp1 vsSimple(model, viewProjection);
		FragmentProgram = compile arbfp1 psSimple(samplerDiffuseBilin);
	}
}

//-------------------------------------------------------------------------------------------------
// opaque
//		Technique used for the non-transparent triangles.
//-------------------------------------------------------------------------------------------------
technique opaque
{
	pass p0
	{
		BlendEnable = false;
		CullFaceEnable = true;
		CullFace = Back;
		DepthMask = true;
		DepthTestEnable = true;

		VertexProgram = compile arbvp1 vsLighting(cameraPosition, time, model, viewProjection, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
		FragmentProgram = compile arbfp1 psLighting(cameraPosition, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
	}
}

//-------------------------------------------------------------------------------------------------
// transparent
//		Technique used for the transparent triangles.
//-------------------------------------------------------------------------------------------------
technique transparent
{
	pass p0
	{
		BlendEnable = false;
		CullFaceEnable = true;
		CullFace = Back;
		DepthMask = true;
		DepthTestEnable = true;
		
		VertexProgram = compile arbvp1 vsLighting(cameraPosition, time, model, viewProjection, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
		FragmentProgram = compile arbfp1 psLighting(cameraPosition, light0position, light0ambient, light0diffuse, light0specular, materialAmbient, materialDiffuse, materialSpecular, materialShininess);
	}
}
